/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.asm

import java.util.Map

import codechicken.asm.transformers.MethodReplacer
import codechicken.asm.{ASMBlock, ASMReader, ModularASMTransformer, ObfMapping}
import net.minecraft.launchwrapper.IClassTransformer

class CubeLibTransformer extends IClassTransformer {

  private val transformer = new ModularASMTransformer("cubelib-transformer")
  private val entity = ASMReader.loadResource("/assets/cubelib/asm/entity.asm")
  transformEntity(entity)

  override def transform(name: String, transformedName: String, basicClass: Array[Byte]): Array[Byte] = {
    return transformer.transform(name, basicClass)
  }

  private def transformEntity(blocks: Map[String, ASMBlock]): Unit = {
    val owner = "net/minecraft/entity/Entity"
    val spawnRunningParticles = new ObfMapping(owner, "func_174830_Y", "()V")
    transformer.add(new MethodReplacer(spawnRunningParticles, blocks.get("entity_n_spawnRunningParticles"), blocks.get("entity_r_spawnRunningParticles")))
  }

}
