/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.gui

import net.minecraft.client.gui.GuiScreen
import net.minecraft.client.renderer.GlStateManager
import net.minecraft.client.renderer.texture.TextureMap
import net.minecraft.util.ResourceLocation
import net.minecraftforge.fml.relauncher.{Side, SideOnly}

/**
  * GuiScreen base class with some usefull helper methods.
  * Note: centerX & centerY are calculated in initGui()!
  */
@SideOnly(Side.CLIENT)
class GuiScreenBase extends GuiScreen {

  protected var centerX = 0
  protected var centerY = 0

  override def initGui: Unit = {
    super.initGui()
    centerX = width / 2
    centerY = height / 2
  }

  def drawTitle(text: String, posX: Int, posY: Int, sizeX: Int): Unit = {
    GlStateManager.pushMatrix()
    val length = fontRenderer.getStringWidth(text)
    drawGradientRect(posX - (sizeX / 2), posY, posX + (sizeX / 2), posY + 24, 0xCC000000, 0x44000000)
    drawHorizontalLine(posX - (sizeX / 2), posX - 1 + (sizeX / 2), posY, 0xFF666666)
    drawHorizontalLine(posX - (sizeX / 2), posX - 1 + (sizeX / 2), posY + 24 - 1, 0xFF666666)
    fontRenderer.drawString(text, posX - (length / 2), posY + 8, 0xFFFFFFFF)
    GlStateManager.popMatrix()
  }

  def drawBackground(location: ResourceLocation, gradient: Boolean): Unit = {
    val texture = mc.getTextureMapBlocks.getAtlasSprite(location.toString)
    GlStateManager.pushMatrix()
    mc.getTextureManager.bindTexture(TextureMap.LOCATION_BLOCKS_TEXTURE)
    for (tileX <- 0 until 64; tileY <- 0 until 32) drawTexturedModalRect(tileX * 16, tileY * 16, texture, 16, 16)
    if (gradient) drawGradient
    GlStateManager.popMatrix()
  }

  def drawGradient: Unit = {
    drawGradientRect(0, 0, width, height / 2, 0xFF000000, 0x33000000)
    drawGradientRect(0, 0, width, height, 0x33000000, 0xFF000000)
  }

}
