/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.particle

import codechicken.lib.render.particle.IModelParticleProvider
import codechicken.lib.texture.TextureUtils
import net.minecraft.client.Minecraft
import net.minecraft.client.particle.{IParticleFactory, Particle}
import net.minecraft.util.math.BlockPos
import net.minecraft.world.World
import net.minecraftforge.fml.relauncher.{Side, SideOnly}

@SideOnly(Side.CLIENT)
class ParticleDiggingCustomFactory extends IParticleFactory {

  override def createParticle(particleID: Int, world: World, x: Double, y: Double, z: Double, xs: Double, ys: Double, zs: Double, params: Int*): Particle = {
    val pos = new BlockPos(params(0), params(1), params(2))
    val state = world.getBlockState(pos)
    val brd = Minecraft.getMinecraft.getBlockRendererDispatcher

    if (state != null && brd != null) {
      val model = brd.getModelForState(state)

      if (model != null) model match {
        case particleProvider: IModelParticleProvider => {
          val iterator = particleProvider.getDestroyEffects(state, world, pos).iterator()

          if(iterator.hasNext) {
            return new ParticleDiggingCustom(world, x, y, z, xs, ys, zs, state, iterator.next()).setBlockPos(pos)
          }
        }
        case _ => return new ParticleDiggingCustom(world, x, y, z, xs, ys, zs, state, TextureUtils.getMissingSprite).setBlockPos(pos)
      }
    }

    return new ParticleDiggingCustom(world, x, y, z, xs, ys, zs, state, TextureUtils.getMissingSprite).setBlockPos(pos)
  }

}
