/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.render

import java.util.{List, Set}

import codechicken.lib.render.particle.IModelParticleProvider
import com.google.common.collect.Sets
import de.keri.cubelib.client.texture.ITextureBlock
import de.keri.cubelib.util.MetadataUtils
import net.minecraft.block.state.IBlockState
import net.minecraft.client.Minecraft
import net.minecraft.client.renderer.block.model.{BakedQuad, IBakedModel, ItemOverrideList}
import net.minecraft.client.renderer.texture.TextureAtlasSprite
import net.minecraft.util.EnumFacing
import net.minecraft.util.math.{BlockPos, RayTraceResult}
import net.minecraft.world.IBlockAccess
import net.minecraftforge.fml.relauncher.{Side, SideOnly}

@SideOnly(Side.CLIENT)
class SimpleBakedModel(quads: List[BakedQuad]) extends IBakedModel with IModelParticleProvider {

  override def getQuads(state: IBlockState, side: EnumFacing, rand: Long): List[BakedQuad] = quads

  override def isGui3d: Boolean = true

  override def getOverrides: ItemOverrideList = ItemOverrideList.NONE

  override def isAmbientOcclusion: Boolean = true

  override def isBuiltInRenderer: Boolean = false

  override def getParticleTexture: TextureAtlasSprite = Minecraft.getMinecraft.getTextureMapBlocks.getMissingSprite

  override def getDestroyEffects(state: IBlockState, world: IBlockAccess, pos: BlockPos): Set[TextureAtlasSprite] = state.getBlock match {
    case block: ITextureBlock if block.getTexture(world, pos, 0) != null => Sets.newHashSet(block.getTexture(world, pos, 0))
    case block: ITextureBlock if block.getTexture(MetadataUtils.getBlockMetadata(world, pos), 0) != null => Sets.newHashSet(block.getTexture(MetadataUtils.getBlockMetadata(world, pos), 0))
    case _ => Sets.newHashSet()
  }

  override def getHitEffects(traceResult: RayTraceResult, state: IBlockState, world: IBlockAccess, pos: BlockPos): Set[TextureAtlasSprite] = state.getBlock match {
    case block: ITextureBlock if block.getTexture(world, pos, 0) != null => Sets.newHashSet(block.getTexture(world, pos, 0))
    case block: ITextureBlock if block.getTexture(MetadataUtils.getBlockMetadata(world, pos), 0) != null => Sets.newHashSet(block.getTexture(MetadataUtils.getBlockMetadata(world, pos), 0))
    case _ => Sets.newHashSet()
  }

}
