/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.render.block

import codechicken.lib.render.block.ICCBlockRenderer
import codechicken.lib.render.item.IItemRenderer
import net.minecraft.block.state.IBlockState
import net.minecraft.client.renderer.block.model.ItemCameraTransforms
import net.minecraft.client.renderer.texture.{TextureAtlasSprite, TextureMap}
import net.minecraft.client.renderer.vertex.DefaultVertexFormats
import net.minecraft.client.renderer.{BufferBuilder, Tessellator}
import net.minecraft.item.ItemStack
import net.minecraft.util.math.BlockPos
import net.minecraft.world.IBlockAccess
import net.minecraftforge.common.model.IModelState
import net.minecraftforge.fml.relauncher.{Side, SideOnly}
import org.lwjgl.opengl.GL11

@SideOnly(Side.CLIENT)
class BlockRenderingAdapter(handler: IBlockRenderingHandler) extends ICCBlockRenderer with IItemRenderer {

  override def renderBlock(world: IBlockAccess, pos: BlockPos, state: IBlockState, buffer: BufferBuilder): Boolean = {
    return handler.renderWorld(world, pos, state, buffer)
  }

  override def handleRenderBlockDamage(world: IBlockAccess, pos: BlockPos, state: IBlockState, sprite: TextureAtlasSprite, buffer: BufferBuilder): Unit = {
    handler.renderDamage(world, pos, state, sprite, buffer)
  }

  override def renderItem(stack: ItemStack, transformType: ItemCameraTransforms.TransformType): Unit = {
    val buffer = Tessellator.getInstance().getBuffer
    buffer.begin(GL11.GL_QUADS, DefaultVertexFormats.ITEM)
    handler.renderInventory(stack, transformType, buffer)
    Tessellator.getInstance().draw()
  }

  override def getTransforms: IModelState = handler.getInventoryTransformations

  override def isGui3d: Boolean = handler.applyItemLighting

  override def isAmbientOcclusion: Boolean = handler.isAmbientOcclusion

  override def renderBrightness(state: IBlockState, brightness: Float): Unit = {}

  override def registerTextures(map: TextureMap): Unit = {}

}
