/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.render.block;

import codechicken.lib.util.TransformUtils;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms.TransformType;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraftforge.common.model.IModelState;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
public interface IBlockRenderingHandler {

    /**
     * Called to render the block in-world.
     * Note that no GL calls are allowed here and that the
     * buffer is already drawing!
     * @param world the current IBlockAccess
     * @param pos the current block position
     * @param state the current IBlockState
     * @param buffer the current BufferBuilder
     * @return true if the block has been rendered comepletely
     */
    boolean renderWorld(IBlockAccess world, BlockPos pos, IBlockState state, BufferBuilder buffer);

    /**
     * Called to render the block damage.
     * Note that no GL calls are allowed here and that the
     * buffer is already drawing!
     * @param world the current IBlockAccess
     * @param pos the current block position
     * @param state the current IBlockState
     * @param texture the current break-texture
     * @param buffer the current BufferBuilder
     */
    void renderDamage(IBlockAccess world, BlockPos pos, IBlockState state, TextureAtlasSprite texture, BufferBuilder buffer);

    /**
     * Called to render the block as an Item.
     * Note that GL calls are allowed here and that the buffer
     * is already drawing!
     * @param stack the current ItemStack
     * @param transformType the current TransformType
     * @param buffer the current BufferBuilder
     */
    void renderInventory(ItemStack stack, TransformType transformType, BufferBuilder buffer);

    /**
     * @return the EnumBlockRenderType assigned to this IBlockRenderingHandler
     */
    EnumBlockRenderType getRenderType();

    /**
     * IModelState used to handle item transformations.
     * @return the transformations for the Item of this Block
     */
    default IModelState getInventoryTransformations(){
        return TransformUtils.DEFAULT_BLOCK;
    }

    /**
     * Basicly the same as GlStateManager.enableLighting() but automatically.
     * @return true if default lighting should be applied to the Item of this Block
     */
    default boolean applyItemLighting(){
        return true;
    }

    /**
     * @return true if the Item of this Block should be rendered with ambient occlusion if enabled
     */
    default boolean isAmbientOcclusion(){
        return true;
    }

}
