/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.render.item;

import codechicken.lib.util.TransformUtils;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms.TransformType;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.model.IModelState;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
public interface IItemRenderingHandler {

    /**
     * Called to render the Item.
     * Note that GL calls are allowed here and that the buffer
     * is already drawing!
     * @param stack the current ItemStack
     * @param transformType the current TransformType
     * @param buffer the current BufferBuilder
     */
    void renderInventory(ItemStack stack, TransformType transformType, BufferBuilder buffer);

    /**
     * @return the EnumItemRenderType assigned to this IItemRenderingHandler
     */
    EnumItemRenderType getRenderType();

    /**
     * IModelState used to handle item transformations.
     * @return the transformations for this Item
     */
    default IModelState getTransformations(){
        return TransformUtils.DEFAULT_ITEM;
    }

    /**
     * Basicly the same as GlStateManager.enableLighting() but automatically.
     * @return true if default lighting should be applied to the Item of this Block
     */
    default boolean applyLighting(){
        return false;
    }

    /**
     * @return true if the Item of this Block should be rendered with ambient occlusion if enabled
     */
    default boolean isAmbientOcclusion(){
        return true;
    }

}
