/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.render.item

import codechicken.lib.util.TransformUtils
import com.google.common.collect.ImmutableList
import de.keri.cubelib.client.texture.ITextureItem
import net.minecraft.client.Minecraft
import net.minecraft.client.renderer.block.model.ItemCameraTransforms
import net.minecraft.client.renderer.vertex.DefaultVertexFormats
import net.minecraft.client.renderer.{BufferBuilder, GlStateManager, Tessellator}
import net.minecraft.item.ItemStack
import net.minecraft.util.ResourceLocation
import net.minecraftforge.client.model.{ItemLayerModel, ModelLoader}
import net.minecraftforge.common.model.{IModelState, TRSRTransformation}
import net.minecraftforge.fml.relauncher.{Side, SideOnly}
import org.lwjgl.opengl.GL11

@SideOnly(Side.CLIENT)
class RenderItemDefault(transformations: IModelState, renderType: EnumItemRenderType) extends IItemRenderingHandler {

  def this(renderType: EnumItemRenderType) = this(TransformUtils.DEFAULT_ITEM, renderType)

  override def renderInventory(stack: ItemStack, transformType: ItemCameraTransforms.TransformType, buffer: BufferBuilder): Unit = {
    Tessellator.getInstance().draw()
    val renderItem = Minecraft.getMinecraft.getRenderItem
    val textureGetter = ModelLoader.defaultTextureGetter()
    val textureHandler = stack.getItem.asInstanceOf[ITextureItem]
    val texture = if (textureHandler.getTexture(stack) != null) textureHandler.getTexture(stack) else textureHandler.getTexture(stack.getMetadata)
    val model = new ItemLayerModel(ImmutableList.of(new ResourceLocation(texture.getIconName)))
    val quads = model.bake(TRSRTransformation.identity(), DefaultVertexFormats.ITEM, textureGetter)
    GlStateManager.pushMatrix()
    GlStateManager.translate(0.5D, 0.5D, 0.5D)
    renderItem.renderItem(stack, quads)
    GlStateManager.popMatrix()
    buffer.begin(GL11.GL_QUADS, DefaultVertexFormats.ITEM)
  }

  override def getRenderType: EnumItemRenderType = renderType

  override def getTransformations: IModelState = transformations

}
