/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.client.texture

import net.minecraft.client.Minecraft
import net.minecraft.client.resources.{IResourceManager, IResourceManagerReloadListener}
import net.minecraftforge.client.event.TextureStitchEvent
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import net.minecraftforge.fml.relauncher.{Side, SideOnly}

import scala.collection.mutable.ArrayBuffer

@SideOnly(Side.CLIENT)
class TextureRegister extends IResourceManagerReloadListener {

  private val handlers = new ArrayBuffer[ITextureRegisterHandler]()

  def registerHandler(handler: ITextureRegisterHandler): Unit = handlers += handler

  @SubscribeEvent
  def onTextureStitch(event: TextureStitchEvent): Unit = handlers.foreach(h => h.registerTextures(event.getMap))

  override def onResourceManagerReload(resourceManager: IResourceManager): Unit = {
    val map = Minecraft.getMinecraft.getTextureMapBlocks
    handlers.foreach(h => h.registerTextures(map))
  }

}
