/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.item

import de.keri.cubelib.client.render.RenderingRegistry
import de.keri.cubelib.client.render.item.{EnumItemRenderType, IItemRenderTypeProvider, RenderItems}
import de.keri.cubelib.client.texture.ITextureItem
import de.keri.cubelib.proxy.ClientProxy
import de.keri.cubeloader.loader.ILoadable
import de.keri.cubeloader.loader.loadstage.{EnumStageType, LoadStage}
import net.minecraft.item.Item
import net.minecraftforge.fml.common.registry.ForgeRegistries
import net.minecraftforge.fml.relauncher.{Side, SideOnly}

trait TDefaultItemImpl extends Item with ILoadable with ITextureItem with IItemRenderTypeProvider {

  @SideOnly(Side.CLIENT)
  override def getRenderType: EnumItemRenderType = RenderItems.DEFAULT

  override def load(loadStage: LoadStage): Unit = {
    loadStage.getStageType match {
      case EnumStageType.PRE_INIT => {
        setModid(loadStage.getModMetadata.getModid)
        setRegistryName(getModid, getItemName)
        setUnlocalizedName(s"$getModid.$getItemName")
        ForgeRegistries.ITEMS.register(this)
      }
      case _ =>
    }
  }

  @SideOnly(Side.CLIENT)
  override def loadClient(loadStage: LoadStage): Unit = {
    loadStage.getStageType match {
      case EnumStageType.PRE_INIT => {
        ClientProxy.getTextureRegister.registerHandler(this)
        RenderingRegistry.registerItem(this)
      }
      case _ =>
    }
  }

  @SideOnly(Side.SERVER)
  override def loadServer(loadStage: LoadStage): Unit = {}

  /**
    * Getter method for retrieving the name this Item is registered to.
    * @return The name this Item is registered to
    */
  def getItemName: String

  /**
    * Getter method for retrieving the modid this Item is registered to.
    * @return The modid this Item is registered to
    */
  def getModid: String

  /**
    * Setter method for setting the name this item is registered to.
    * @param modid The name this Item is registered to
    */
  def setModid(modid: String): Unit

}
