/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.proxy

import codechicken.lib.packet.PacketCustom
import de.keri.cubelib.client.particle.Particles
import de.keri.cubelib.client.render.block.RenderBlocks
import de.keri.cubelib.client.render.item.RenderItems
import de.keri.cubelib.client.texture.TextureRegister
import de.keri.cubelib.network.CubeLibCPH
import de.keri.cubeloader.loader.IProxy
import de.keri.cubeloader.loader.loadstage.{EnumStageType, LoadStage}
import net.minecraft.client.Minecraft
import net.minecraft.client.resources.IReloadableResourceManager
import net.minecraftforge.common.MinecraftForge

class ClientProxy extends IProxy {

  override def load(loadStage: LoadStage): Unit = loadStage.getStageType match {
    case EnumStageType.PRE_INIT => {
      RenderBlocks.initializeRenderers()
      RenderItems.initializeRenderers()
      ClientProxy.TEXTURE_REGISTER = new TextureRegister
      MinecraftForge.EVENT_BUS.register(ClientProxy.TEXTURE_REGISTER)
    }
    case EnumStageType.INIT => {
      Minecraft.getMinecraft.getResourceManager.asInstanceOf[IReloadableResourceManager].registerReloadListener(ClientProxy.TEXTURE_REGISTER)
      PacketCustom.assignHandler("cubelib", new CubeLibCPH)
      Particles.registerParticles()
    }
    case _ =>
  }

}

object ClientProxy {

  private var TEXTURE_REGISTER: TextureRegister = _

  def getTextureRegister: TextureRegister = TEXTURE_REGISTER

}
