/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.recipe

import de.keri.cubelib.recipe.ShapedOreRecipe.VanillaShapedRecipe
import de.keri.cubelib.util.ItemStackUtils
import de.keri.cubelib.util.ScalaAdapters._
import de.keri.cubeloader.loader.ILoadable
import de.keri.cubeloader.loader.loadstage.{EnumStageType, LoadStage}
import net.minecraft.block.Block
import net.minecraft.item.crafting.Ingredient
import net.minecraft.item.{Item, ItemStack}
import net.minecraft.util.{NonNullList, ResourceLocation}
import net.minecraftforge.fml.common.registry.ForgeRegistries
import net.minecraftforge.fml.relauncher.{Side, SideOnly}
import net.minecraftforge.oredict.OreDictionary

import scala.collection.mutable.{ArrayBuffer, HashMap}

class ShapedOreRecipe(output: ItemStack, params: Any*) extends ILoadable {

  val ingredients = new ArrayBuffer[String]()
  val ingredientMap = new HashMap[Char, ArrayBuffer[ItemStack]]()
  val ingredientList = NonNullList.create[Ingredient]()
  var width = 0
  var hasMissingIngredients = false
  var isPatternDefined = false

  for (paramIndex <- 0 until params.length) params(paramIndex) match {
    case s: String if !isPatternDefined => ingredients += s
    case c: Char => {
      if (!isPatternDefined) isPatternDefined = true

      params(paramIndex + 1) match {
        case stack: ItemStack => putToMapBuffer(ingredientMap, c, stack)
        case item: Item => putToMapBuffer(ingredientMap, c, new ItemStack(item, 1, 0))
        case block: Block => putToMapBuffer(ingredientMap, c, new ItemStack(block, 1, 0))
        case s: String => {
          val items = OreDictionary.getOres(s)
          for (i <- 0 until items.size()) putToMapBuffer(ingredientMap, c, items.get(i))
        }
        case _ =>
      }
    }
    case _ =>
  }

  ingredients.foreach(row => {
    if (width < row.length) width = row.length
    for (i <- 0 until row.length) {
      if (row.charAt(i) != ' ') {
        val ingredient = ingredientMap.get(row.charAt(i))
        if (ingredient.isDefined) ingredientList.add(IngredientHelper.fromStackArray(ingredient.get.toArray[ItemStack]))
        else hasMissingIngredients = true
      }
      else ingredientList.add(Ingredient.fromStacks(ItemStackUtils.getEmptyStack))
    }
  })

  override def load(loadStage: LoadStage): Unit = loadStage.getStageType match {
    case EnumStageType.INIT if !hasMissingIngredients => {
      val recipe = new VanillaShapedRecipe(loadStage.getModMetadata.getModid, width, ingredients.size, ingredientList, output)
      recipe.setRegistryName(new ResourceLocation(loadStage.getModMetadata.getModid, s"shaped_ore_recipe_${ShapedOreRecipe.recipeIndex}"))
      ForgeRegistries.RECIPES.register(recipe)
      ShapedOreRecipe.recipeIndex += 1
    }
    case _ =>
  }

  @SideOnly(Side.CLIENT)
  override def loadClient(loadStage: LoadStage): Unit = {}

  @SideOnly(Side.SERVER)
  override def loadServer(loadStage: LoadStage): Unit = {}

}

object ShapedOreRecipe {

  private final type VanillaShapedRecipe = net.minecraft.item.crafting.ShapedRecipes
  private var recipeIndex = 0

}
