/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.recipe

import de.keri.cubelib.recipe.ShapelessOreRecipe.VanillaShapelessRecipe
import de.keri.cubeloader.loader.ILoadable
import de.keri.cubeloader.loader.loadstage.{EnumStageType, LoadStage}
import net.minecraft.block.Block
import net.minecraft.item.crafting.Ingredient
import net.minecraft.item.{Item, ItemStack}
import net.minecraft.util.{NonNullList, ResourceLocation}
import net.minecraftforge.fml.common.registry.ForgeRegistries
import net.minecraftforge.fml.relauncher.{Side, SideOnly}
import net.minecraftforge.oredict.OreDictionary

class ShapelessOreRecipe(output: ItemStack, params: Any*) extends ILoadable {

  val ingredientList = NonNullList.create[Ingredient]()

  for (param <- params) param match {
    case stack: ItemStack => ingredientList.add(Ingredient.fromStacks(stack))
    case item: Item => ingredientList.add(Ingredient.fromStacks(new ItemStack(item, 1, 0)))
    case block: Block => ingredientList.add(Ingredient.fromStacks(new ItemStack(block, 1, 0)))
    case s: String => ingredientList.add(IngredientHelper.fromStackList(OreDictionary.getOres(s)))
    case _ =>
  }

  override def load(loadStage: LoadStage): Unit = loadStage.getStageType match {
    case EnumStageType.INIT => {
      val recipe = new VanillaShapelessRecipe(loadStage.getModMetadata.getModid, output, ingredientList)
      recipe.setRegistryName(new ResourceLocation(loadStage.getModMetadata.getModid, s"shapeless_ore_recipe_${ShapelessOreRecipe.recipeIndex}"))
      ForgeRegistries.RECIPES.register(recipe)
      ShapelessOreRecipe.recipeIndex += 1
    }
    case _ =>
  }

  @SideOnly(Side.CLIENT)
  override def loadClient(loadStage: LoadStage): Unit = {}

  @SideOnly(Side.SERVER)
  override def loadServer(loadStage: LoadStage): Unit = {}

}

object ShapelessOreRecipe {

  private final type VanillaShapelessRecipe = net.minecraft.item.crafting.ShapelessRecipes
  private var recipeIndex = 0

}
