/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.util

import java.util.List

import de.keri.cubelib.client.render.SimpleBakedModel
import net.minecraft.block.state.IBlockState
import net.minecraft.client.Minecraft
import net.minecraft.client.renderer.BufferBuilder
import net.minecraft.client.renderer.block.model.BakedQuad
import net.minecraft.client.renderer.vertex.{DefaultVertexFormats, VertexFormat}
import net.minecraft.util.math.{BlockPos, MathHelper, Vec3i}
import net.minecraft.world.IBlockAccess
import net.minecraftforge.common.ForgeModContainer
import net.minecraftforge.fml.client.FMLClientHandler
import net.minecraftforge.fml.relauncher.{Side, SideOnly}
import org.lwjgl.opengl.GL11

@SideOnly(Side.CLIENT)
object RenderUtils {

  private final val ITEM_WITH_LMAP = new VertexFormat(DefaultVertexFormats.ITEM).addElement(DefaultVertexFormats.TEX_2S)

  def getFormatWithLightmap(format: VertexFormat): VertexFormat = {
    if(FMLClientHandler.instance().hasOptifine || !ForgeModContainer.forgeLightPipelineEnabled) format

    format match {
      case DefaultVertexFormats.BLOCK => DefaultVertexFormats.BLOCK
      case DefaultVertexFormats.ITEM => ITEM_WITH_LMAP
      case f if !f.hasUvOffset(1) => new VertexFormat(format).addElement(DefaultVertexFormats.TEX_2S)
      case _ => format
    }
  }

  def renderQuads(buffer: BufferBuilder, world: IBlockAccess, pos: BlockPos, state: IBlockState, quads: List[BakedQuad]): Boolean = {
    val useAo = Minecraft.isAmbientOcclusionEnabled
    val model = new SimpleBakedModel(quads)
    val modelRenderer = Minecraft.getMinecraft.getBlockRendererDispatcher.getBlockModelRenderer
    val random = MathHelper.getPositionRandom(new Vec3i(pos.getX, pos.getY, pos.getZ))
    if(useAo) modelRenderer.renderModelSmooth(world, model, state, pos, buffer, true, random)
    else modelRenderer.renderModelFlat(world, model, state, pos, buffer, true, random)
  }

  def disableMipmap: Array[Int] = {
    val minFilter = GL11.glGetTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER)
    val magFilter = GL11.glGetTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER)
    GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST)
    GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST)
    Array(minFilter, magFilter)
  }

  def enableMipmap(data: Array[Int]): Unit = {
    GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, data(0))
    GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, data(1))
  }

}
