/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubelib.util

import java.util.function.Consumer
import java.util.{List, Set}

import com.google.common.collect.Lists
import org.apache.commons.lang3.tuple.{Pair, Triple}

import scala.collection.mutable.{ArrayBuffer, HashMap}

object ScalaAdapters {

  def traverse[T](set: Set[T]): Traversable[T] = {
    val buffer = new ArrayBuffer[T]()
    val iterator = set.iterator()
    while (iterator.hasNext) buffer += iterator.next()
    return buffer
  }

  def traverse[T](list: List[T]): Traversable[T] = {
    val buffer = new ArrayBuffer[T]()
    val iterator = list.iterator()
    while (iterator.hasNext) buffer += iterator.next()
    return buffer
  }

  def toList[T](buffer: ArrayBuffer[T]): List[T] = {
    val list = Lists.newArrayList[T]()
    buffer.foreach(e => list.add(e))
    return list
  }

  def toBuffer[T](list: List[T]): ArrayBuffer[T] = {
    val buffer = new ArrayBuffer[T]()
    val iterator = list.iterator()
    while (iterator.hasNext) buffer += iterator.next()
    return buffer
  }

  def tupleToPair[A, B](tuple: (A, B)): Pair[A, B] = Pair.of(tuple._1, tuple._2)

  def pairToTuple[A, B](pair: Pair[A, B]): (A, B) = (pair.getLeft, pair.getRight)

  def tupleToTriple[A, B, C](tuple: (A, B, C)): Triple[A, B, C] = Triple.of(tuple._1, tuple._2, tuple._3)

  def tripleToTuple[A, B, C](triple: Triple[A, B, C]): (A, B, C) = (triple.getLeft, triple.getMiddle, triple.getRight)

  def forEach[T](buffer: ArrayBuffer[T], consumer: Consumer[T]): Unit = buffer.foreach(e => consumer.accept(e))

  def newArrayBuffer[T](elements: T*): ArrayBuffer[T] = {
    val buffer = new ArrayBuffer[T]()
    for (e <- elements) buffer += e
    return buffer
  }

  def putToMapBuffer[K, V](map: HashMap[K, ArrayBuffer[V]], key: K, element: V): Unit = {
    if (map.contains(key)) map.apply(key) += element
    else map.put(key, newArrayBuffer[V](element))
  }

}
