/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.client

import de.keri.cubeloader.CubeLoader
import de.keri.cubeloader.client.gui.GuiModlist
import net.minecraft.client.Minecraft
import net.minecraft.client.gui.{GuiButton, GuiMainMenu}
import net.minecraftforge.client.event.GuiScreenEvent.{ActionPerformedEvent, InitGuiEvent}
import net.minecraftforge.fml.client.FMLClientHandler
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import net.minecraftforge.fml.common.gameevent.TickEvent.{ClientTickEvent, Phase, RenderTickEvent}
import net.minecraftforge.fml.relauncher.{Side, SideOnly}

@SideOnly(Side.CLIENT)
class ClientEventHandler {

  @SubscribeEvent
  def onInitGui(event: InitGuiEvent): Unit = if (event.getGui.getClass == classOf[GuiMainMenu]) {
    val posX = event.getGui.width / 2 - 124
    val posY = event.getGui.height / 4 + 96
    event.getButtonList.add(new GuiButton(CubeLoader.CONFIG.buttonId, posX, posY, 20, 20, "C"))
  }

  @SubscribeEvent
  def onActionPerformed(event: ActionPerformedEvent): Unit = if (event.getGui.getClass == classOf[GuiMainMenu]) {
    val player = Minecraft.getMinecraft.player
    if (event.getButton.id == CubeLoader.CONFIG.buttonId) FMLClientHandler.instance().displayGuiScreen(player, new GuiModlist)
  }

  @SubscribeEvent
  def onClientTick(event: ClientTickEvent): Unit = if (event.phase == Phase.END) ClientEventHandler.RENDER_TICK += 1

  @SubscribeEvent
  def onRenderTick(event: RenderTickEvent): Unit = if (event.phase == Phase.END) ClientEventHandler.RENDER_FRAME = event.renderTickTime

}

@SideOnly(Side.CLIENT)
object ClientEventHandler {

  private var RENDER_TICK = 0
  private var RENDER_FRAME = 0F

  def getRenderTime: Int = RENDER_TICK + RENDER_FRAME.asInstanceOf[Int]

}
