/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.client.gui

import com.google.common.collect.Lists
import de.keri.cubelib.client.gui.GuiScreenBase
import de.keri.cubelib.util.{CommonUtils, ItemStackUtils}
import de.keri.cubeloader.CubeLoader
import de.keri.cubeloader.client.ClientEventHandler
import de.keri.cubeloader.loader.ModLoader
import net.minecraft.client.gui.{GuiButton, GuiMainMenu}
import net.minecraft.client.model.ModelHumanoidHead
import net.minecraft.client.renderer.entity.Render
import net.minecraft.client.renderer.{GlStateManager, RenderHelper, Tessellator}
import net.minecraft.entity.item.EntityItem
import net.minecraft.util.ResourceLocation
import net.minecraftforge.fml.client.config.GuiUtils
import net.minecraftforge.fml.client.{FMLClientHandler, GuiScrollingList}
import net.minecraftforge.fml.relauncher.{Side, SideOnly}
import org.lwjgl.input.Mouse

@SideOnly(Side.CLIENT)
class GuiModlist extends GuiScreenBase {

  private var modlist: GuiScrollingList = _
  private var selectedMod = 0

  override def drawScreen(mouseX: Int, mouseY: Int, partialTicks: Float): Unit = {
    drawBackground(new ResourceLocation("minecraft", "blocks/glazed_terracotta_red"), true)
    drawTitle("Loaded Mods", width / 2, 10, 256)
    if (modlist != null) modlist.drawScreen(mouseX, mouseY, partialTicks)
    GlStateManager.pushMatrix()
    drawHorizontalLine(16, width - 17, 42, 0xFF666666)
    drawHorizontalLine(16, width - 17, height - 48, 0xFF666666)
    GlStateManager.popMatrix()
    super.drawScreen(mouseX, mouseY, partialTicks)
  }

  override def initGui: Unit = {
    super.initGui()
    addButton(new GuiButton(0, centerX - 32, centerY + 90, 64, 20, "Done"))
    val buttonWebsite = new GuiButton(1, centerX - 100, centerY + 90, 64, 20, "Website")
    buttonWebsite.enabled = ModLoader.getMetadata(ModLoader.getLoadedMods(selectedMod).getModid).getWebsite != null
    addButton(buttonWebsite)
    val buttonConfig = new GuiButton(2, centerX + 36, centerY + 90, 64, 20, "Config")
    buttonConfig.enabled = false
    addButton(buttonConfig)
    addButton(new GuiButton(3, centerX + 132, centerY - 115, 20, 20, "?"))

    modlist = new GuiScrollingList(mc, width - 32, height, 42, height - 48, 16, 48, width, height) {
      override def drawBackground(): Unit = {}

      override def elementClicked(index: Int, doubleClick: Boolean): Unit = if (selectedMod != index) selectedMod = index

      override def drawSlot(slotIdx: Int, entryRight: Int, slotTop: Int, slotBuffer: Int, tess: Tessellator): Unit = {
        val mod = ModLoader.getLoadedMods(slotIdx)
        if (mod == null) return
        val metadata = ModLoader.getMetadata(mod.getModid)
        val icon = ItemStackUtils.getStackFromName(metadata.getIcon)
        GlStateManager.pushMatrix()
        fontRenderer.drawString(fontRenderer.trimStringToWidth(metadata.getName, listWidth - 10), left + 3, slotTop + 2, 0xFFFFFFFF)
        fontRenderer.drawString(fontRenderer.trimStringToWidth(metadata.getVersion, listWidth - (5 + slotBuffer)), left + 3, slotTop + 12, 0xFFCCCCCC)
        fontRenderer.drawString(fontRenderer.trimStringToWidth("Authors:", listWidth - (5 + slotBuffer)), left + 3, slotTop + 28, 0xFFCCCCCC)
        GlStateManager.popMatrix()
        GlStateManager.pushMatrix()
        GlStateManager.enableLighting()
        GlStateManager.translate(entryRight - 128D, slotTop - 38D, 0D)
        GlStateManager.translate(100D, 100D, 0D)
        GlStateManager.rotate(180F, 0F, 0F, 1F)
        GlStateManager.rotate(ClientEventHandler.getRenderTime * 4, 0F, 1F, 0F)
        GlStateManager.translate(-100D, -100D, 0D)
        GlStateManager.scale(100D, 100D, 100D)
        val renderer: Render[EntityItem] = mc.getRenderManager.getEntityClassRenderObject(classOf[EntityItem])
        val entity = new EntityItem(mc.world, 0D, 0D, 0D)
        entity.setAgeToCreativeDespawnTime()
        entity.hoverStart = 0F
        entity.setItem(icon)
        renderer.doRender(entity, 1D, 1D, 0D, 0F, 0F)
        GlStateManager.disableLighting()
        GlStateManager.popMatrix()
        val model = new ModelHumanoidHead

        for (i <- metadata.getAuthors.indices) {
          val tX = left + 58D + i * 20D
          val tY = slotTop + 40D
          GlStateManager.pushMatrix()
          GlStateManager.translate(tX, tY, 0D)
          mc.getTextureManager.bindTexture(CubeLoader.PROXY.getAuthorSkin(metadata.getAuthors(i).getId))
          model.render(null, 0F, 0F, 0F, 0F, 0F, 2F)
          GlStateManager.translate(-tX, -tY, 0D)
          GlStateManager.popMatrix()
        }

        if (selectedMod == slotIdx) {
          for (i <- metadata.getAuthors.indices) {
            val tX = left + 58D + i * 20D
            val tY = slotTop + 40D

            if (mouseX >= tX - 8D && mouseX <= tX + 8D && mouseY >= tY - 16D && mouseY <= tY) {
              GlStateManager.pushMatrix()
              GuiUtils.drawHoveringText(Lists.newArrayList[String](metadata.getAuthors(i).getName), mouseX, mouseY, screenWidth, screenHeight, 200, fontRenderer)
              GlStateManager.disableLighting()
              GlStateManager.disableRescaleNormal()
              RenderHelper.disableStandardItemLighting()
              GlStateManager.popMatrix()
            }
          }
        }
      }

      override def isSelected(index: Int): Boolean = selectedMod == index

      override def getSize: Int = ModLoader.getLoadedMods.size
    }
  }

  override def actionPerformed(button: GuiButton): Unit = {
    super.actionPerformed(button)
    if (modlist != null) modlist.actionPerformed(button)

    button.id match {
      case 0 => FMLClientHandler.instance().displayGuiScreen(mc.player, new GuiMainMenu)
      case 1 => CommonUtils.openBrowser(ModLoader.getMetadata(ModLoader.getLoadedMods(selectedMod).getModid).getWebsite)
      case 2 => //mod config
      case 3 => FMLClientHandler.instance().displayGuiScreen(mc.player, new GuiVersionInfo)
      case _ =>
    }
  }

  override def handleMouseInput: Unit = {
    super.handleMouseInput()
    val mouseX = Mouse.getEventX * width / mc.displayWidth
    val mouseY = height - Mouse.getEventY * height / mc.displayHeight - 1
    if (modlist != null) modlist.handleMouseInput(mouseX, mouseY)
  }

}
