/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.client.gui

import de.keri.cubelib.client.gui.GuiScreenBase
import de.keri.cubeloader.loader.ModLoader
import de.keri.cubeloader.util.CubeLoaderVersion
import net.minecraft.client.gui.GuiButton
import net.minecraft.client.renderer.GlStateManager
import net.minecraft.util.ResourceLocation
import net.minecraft.util.text.TextFormatting
import net.minecraftforge.common.ForgeVersion
import net.minecraftforge.fml.client.FMLClientHandler
import net.minecraftforge.fml.common.Loader
import net.minecraftforge.fml.relauncher.{Side, SideOnly}

import scala.collection.mutable.ArrayBuffer

@SideOnly(Side.CLIENT)
class GuiVersionInfo extends GuiScreenBase {

  private val info = new ArrayBuffer[String]()

  override def drawScreen(mouseX: Int, mouseY: Int, partialTicks: Float): Unit = {
    drawBackground(new ResourceLocation("minecraft", "blocks/glazed_terracotta_red"), true)
    drawTitle("Version Info", width / 2, 10, 256)

    GlStateManager.pushMatrix()
    val sizeX = 314
    val sizeY = 158
    val posX = centerX - (sizeX / 2)
    val posY = centerY - (sizeY / 2) - 4
    drawGradientRect(posX, posY, posX + sizeX, posY + sizeY, 0xCC000000, 0x44000000)
    drawHorizontalLine(posX, posX + sizeX - 1, posY, 0xFF666666)
    drawHorizontalLine(posX, posX + sizeX - 1, posY + sizeY - 1, 0xFF666666)

    for (i <- info.indices) {
      val length = fontRenderer.getStringWidth(info(i))
      val textY = centerY - 68 + (i * 12)
      fontRenderer.drawString(info(i), centerX - (length / 2), textY, 0xFFFFFFFF)
    }

    GlStateManager.popMatrix()

    super.drawScreen(mouseX, mouseY, partialTicks)
  }

  override def initGui: Unit = {
    super.initGui

    if (info.isEmpty) {
      info += s"${TextFormatting.GREEN}Minecraft Version: ${mc.getVersion}"
      info += s"${TextFormatting.AQUA}MCP Version: ${Loader.instance().getMCPVersionString}"
      info += s"${TextFormatting.AQUA}FML Version: ${Loader.instance().getFMLVersionString}"
      info += s"${TextFormatting.AQUA}Forge Version: ${ForgeVersion.getVersion}"
      info += s"CubeLoader Version: ${CubeLoaderVersion.VERSION}"
      info += s"CubeLib Version: ${ModLoader.getMetadata("cubelib").getVersion}"
    }

    addButton(new GuiButton(0, centerX - 32, centerY + 84, 64, 20, "Done"))
  }

  override def actionPerformed(button: GuiButton): Unit = {
    super.actionPerformed(button)

    button.id match {
      case 0 => FMLClientHandler.instance().displayGuiScreen(mc.player, new GuiModlist)
      case _ =>
    }
  }

}
