/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.loader

import java.util
import java.util.regex.Pattern

import com.google.common.eventbus.EventBus
import de.keri.cubeloader.loader.Bootstrap._
import de.keri.cubeloader.util.CubeLoaderVersion
import net.minecraftforge.fml.common.discovery.ModDiscoverer
import net.minecraftforge.fml.common.{DummyModContainer, LoadController, Loader}
import net.minecraftforge.fml.relauncher.{IFMLLoadingPlugin, ReflectionHelper}

class Bootstrap extends DummyModContainer(new FMLModMetadata) with IFMLLoadingPlugin {

  private val metadata = getMetadata
  metadata.modId = "cubeloader-bootstrap"
  metadata.name = "CubeLoader Bootstrap"
  metadata.version = CubeLoaderVersion.VERSION
  metadata.parent = "cubeloader"
  Bootstrap.bootstrapEarly()

  override def getASMTransformerClass: Array[String] = Array("de.keri.cubelib.asm.CubeLibTransformer")

  override def injectData(data: util.Map[String, AnyRef]): Unit = {}

  override def getAccessTransformerClass: String = null

  override def getModContainerClass: String = "de.keri.cubeloader.loader.Bootstrap"

  override def getSetupClass: String = null

  override def registerBus(bus: EventBus, controller: LoadController): Boolean = {
    bus.register(this)
    return true
  }

}

object Bootstrap {

  private final type FMLModMetadata = net.minecraftforge.fml.common.ModMetadata
  private final val FILE_PATTERN = Pattern.compile("(.+).(zip|jar|cubemod)$")
  private var MOD_DISCOVERER: ModDiscoverer = _

  def bootstrapEarly(): Unit = {
    setStaticField(classOf[ModDiscoverer], "zipJar", FILE_PATTERN, true)
  }

  def bootstrap(): Unit = {
    MOD_DISCOVERER = ReflectionHelper.getPrivateValue(classOf[Loader], Loader.instance(), "discoverer").asInstanceOf[ModDiscoverer]
  }

  private def setStaticField(clazz: Class[_], name: String, value: AnyRef, forceAccess: Boolean): Unit = {
    val field = clazz.getDeclaredField(name)
    if (forceAccess) field.setAccessible(true)
    field.set(null, value)
  }

  def getModDiscoverer: ModDiscoverer = MOD_DISCOVERER

}
