/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.loader

import com.google.common.collect.Lists
import com.google.common.eventbus.EventBus
import de.keri.cubeloader.util.CubeLoaderVersion
import net.minecraftforge.fml.common.versioning.{ArtifactVersion, DefaultArtifactVersion}
import net.minecraftforge.fml.common.{DummyModContainer, LoadController}

class CubeModContainer(metadata: ModMetadata) extends DummyModContainer(new CubeModContainer.ForgeMetadata) {

  val meta = getMetadata
  meta.modId = metadata.getModid
  meta.name = metadata.getName
  meta.version = metadata.getVersion
  val authorList = Lists.newArrayList[String]()
  metadata.getAuthors.foreach(a => authorList.add(a.getName))
  meta.authorList = authorList
  val deps = Lists.newArrayList[ArtifactVersion](new DefaultArtifactVersion("cubeloader", CubeLoaderVersion.VERSION))
  meta.dependencies = deps
  meta.parent = "cubeloader"

  override def registerBus(bus: EventBus, controller: LoadController): Boolean = {
    bus.register(this)
    return true
  }

}

object CubeModContainer {

  private final type ForgeMetadata = net.minecraftforge.fml.common.ModMetadata

}
