/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.loader.loadstage;

import net.minecraftforge.fml.common.event.*;

public enum EnumStageType {

    CONSTRUCTION(0, "Construction", FMLConstructionEvent.class),
    PRE_INIT(1, "PreInitialization", FMLPreInitializationEvent.class),
    INIT(2, "Initialization", FMLInitializationEvent.class),
    POST_INIT(3, "PostInitialization", FMLPostInitializationEvent.class),
    SERVER_ABOUT_TO_START(4, "ServerAboutToStart", FMLServerAboutToStartEvent.class),
    SERVER_STARTING(5, "ServerStarting", FMLServerStartingEvent.class),
    SERVER_STARTED(6, "ServerStarted", FMLServerStartedEvent.class),
    SERVER_STOPPING(7, "ServerStopping", FMLServerStoppingEvent.class),
    SERVER_STOPPED(8, "ServerStopped", FMLServerStoppedEvent.class);

    private int index;
    private String name;
    private Class<? extends FMLEvent> eventClass;

    EnumStageType(int index, String name, Class<? extends FMLEvent> eventClass){
        this.index = index;
        this.name = name;
        this.eventClass = eventClass;
    }

    public int getIndex(){
        return this.index;
    }

    public String getName(){
        return this.name;
    }

    public Class<? extends FMLEvent> getEventClass(){
        return this.eventClass;
    }

    public static EnumStageType getTypeFromClass(Class<? extends FMLEvent> eventClass){
        EnumStageType type = null;

        for(int i = 0; i < values().length; i++){
            if(values()[i].getEventClass() == eventClass){
                type = values()[i];
                break;
            }
        }

        return type;
    }

}
