/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.loader.loadstage

import java.io.File
import java.lang.annotation.Annotation
import java.util.{ArrayList, List}

import de.keri.cubelib.util.ScalaAdapters._
import de.keri.cubeloader.loader.{ILoadable, IMod, ModLoader, ModMetadata}
import net.minecraftforge.fml.common.event.{FMLEvent, FMLPreInitializationEvent}

import scala.collection.mutable.ArrayBuffer

class LoadStage(event: FMLEvent, stageType: EnumStageType, mod: IMod) {

  private val loadables = new ArrayBuffer[ILoadable]()

  def getStageType: EnumStageType = stageType

  def getModMetadata: ModMetadata = ModLoader.getMetadata(mod.getModid)

  def getLoadablesAsBuffer: ArrayBuffer[ILoadable] = loadables

  def getLoadablesAsList: List[ILoadable] = toList(loadables)

  def registerLoadable(loadable: ILoadable): Unit = loadables += loadable

  def getConfigFile: File = stageType match {
      case EnumStageType.PRE_INIT => event.asInstanceOf[FMLPreInitializationEvent].getSuggestedConfigurationFile
      case _ => throw new LoadStageException(s"getConfigFile called in the wrong StageType: ${stageType.getName}")
  }

  def getClassesAsBuffer(annotation: Class[_ <: Annotation]): ArrayBuffer[Class[_ <: _]] = {
    val classes = new ArrayBuffer[Class[_ <: _]]()

    stageType match {
      case EnumStageType.PRE_INIT => {
        val preInitEvent = event.asInstanceOf[FMLPreInitializationEvent]
        val dataIterator = preInitEvent.getAsmData.getAll(annotation.getName).iterator()
        while(dataIterator.hasNext) classes += Class.forName(dataIterator.next().getClassName)
      }
      case _ => throw new LoadStageException(s"getClassesForAnnotation called in the wrong StageType: ${stageType.getName}")
    }

    classes
  }

  def getClassesAsList(annotation: Class[_ <: Annotation]): List[Class[_ <: _]] = {
    val classes = new ArrayList[Class[_ <: _]]()

    stageType match {
      case EnumStageType.PRE_INIT => {
        val preInitEvent = event.asInstanceOf[FMLPreInitializationEvent]
        val dataIterator = preInitEvent.getAsmData.getAll(annotation.getName).iterator()
        while(dataIterator.hasNext) classes.add(Class.forName(dataIterator.next().getClassName))
      }
      case _ => throw new LoadStageException(s"getClassesForAnnotation called in the wrong StageType: ${stageType.getName}")
    }

    classes
  }

}
