/*
 * This file is part of CubeLoader.
 * Copyright (c) 2016 - 2017, KitsuneAlex, All rights reserved.
 *
 * CubeLoader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CubeLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with CubeLoader.  If not, see <http://www.gnu.org/licenses/lgpl>.
 */

package de.keri.cubeloader.proxy

import java.io.File
import java.util.{List, UUID}

import com.mojang.authlib.minecraft.MinecraftProfileTexture
import de.keri.cubeloader.client.ClientEventHandler
import de.keri.cubeloader.loader.{Bootstrap, ModLoader, ModMetadata}
import de.keri.cubeloader.util.CubeLogger
import net.minecraft.client.Minecraft
import net.minecraft.client.resources.{DefaultPlayerSkin, FileResourcePack, FolderResourcePack, IResourcePack}
import net.minecraft.launchwrapper.Launch
import net.minecraft.util.ResourceLocation
import net.minecraftforge.common.MinecraftForge
import net.minecraftforge.fml.client.FMLClientHandler
import net.minecraftforge.fml.common.ModContainer
import net.minecraftforge.fml.common.event._
import net.minecraftforge.fml.relauncher.ReflectionHelper

import scala.collection.mutable.HashMap

class ClientProxy extends CommonProxy {

  override def construction(event: FMLConstructionEvent): Unit = {
    super.construction(event)
    injectResources()
    ModLoader.handleConstructionClient(event)
  }

  override def preInit(event: FMLPreInitializationEvent): Unit = {
    super.preInit(event)
    ModLoader.handlePreInitClient(event)
    MinecraftForge.EVENT_BUS.register(new ClientEventHandler)
  }

  override def init(event: FMLInitializationEvent): Unit = {
    super.init(event)
    ModLoader.handleInitClient(event)
    if (ClientProxy.IS_OFFLINE) CubeLogger.error("Either you are offline or you're a dickhead and should buy the game!")
  }

  override def postInit(event: FMLPostInitializationEvent): Unit = {
    super.postInit(event)
    ModLoader.handlePostInitClient(event)
    ModLoader.getLoadedMods.foreach(m => loadAuthorsSkins(ModLoader.getMetadata(m.getModid)).foreach(s => ClientProxy.SKIN_CACHE += s))
  }

  override def serverAboutToStart(event: FMLServerAboutToStartEvent): Unit = {
    super.serverAboutToStart(event)
    ModLoader.handleServerAboutToStartClient(event)
  }

  override def serverStarting(event: FMLServerStartingEvent): Unit = {
    super.serverStarting(event)
    ModLoader.handleServerStartingClient(event)
  }

  override def serverStarted(event: FMLServerStartedEvent): Unit = {
    super.serverStarted(event)
    ModLoader.handleServerStartedClient(event)
  }

  override def serverStopping(event: FMLServerStoppingEvent): Unit = {
    super.serverStopping(event)
    ModLoader.handleServerStoppingClient(event)
  }

  override def serverStopped(event: FMLServerStoppedEvent): Unit = {
    super.serverStopped(event)
    ModLoader.handleServerStoppedClient(event)
  }

  override def loadAuthorsSkins(metadata: ModMetadata): HashMap[UUID, ResourceLocation] = {
    val skinManager = Minecraft.getMinecraft.getSkinManager
    val session = Minecraft.getMinecraft.getSessionService
    val textures = new HashMap[UUID, ResourceLocation]()

    metadata.getAuthors.foreach(author => {
      val profile = session.fillProfileProperties(author, false)
      val map = skinManager.loadSkinFromCache(profile)

      if (map != null && !map.isEmpty) {
        val profileTexture = map.get(MinecraftProfileTexture.Type.SKIN)
        textures.put(author.getId, skinManager.loadSkin(profileTexture, MinecraftProfileTexture.Type.SKIN))
      }
      else {
        ClientProxy.IS_OFFLINE = true
        textures.put(author.getId, DefaultPlayerSkin.getDefaultSkinLegacy)
      }
    })

    return textures
  }

  override def getAuthorSkin(uuid: UUID): ResourceLocation = ClientProxy.SKIN_CACHE.apply(uuid)

  override def registerContainer(container: ModContainer): Unit = {
    super.registerContainer(container)
  }

  private def injectResources(): Unit = {
    val packs = ReflectionHelper.getPrivateValue(classOf[FMLClientHandler], FMLClientHandler.instance(), "resourcePackList").asInstanceOf[List[IResourcePack]]

    if (Launch.blackboard.get("fml.deobfuscatedEnvironment").asInstanceOf[Boolean]) {
      val file = new File(getClass.getResource("/").getFile)
      packs.add(new FolderResourcePack(file))
    }

    for (i <- 0 until Bootstrap.getModDiscoverer.getNonModLibs.size()) {
      val file = Bootstrap.getModDiscoverer.getNonModLibs.get(i)

      if (file.getName.contains(".cubemod")) {
        packs.add(new FileResourcePack(file))
      }
    }
  }

}

object ClientProxy {

  private val SKIN_CACHE = new HashMap[UUID, ResourceLocation]()
  private var IS_OFFLINE = false

}
